<?php

namespace PW\PWSMS\Gateways;

use PW\PWSMS\PWSMS;
use SoapClient;
use SoapFault;

class Asanak implements GatewayInterface {
    use GatewayTrait;

    public static function id() {
        return 'asanak';
    }

    public static function name() {
        return 'asanak.com';
    }

    public function send() {
        $username = $this->username;
        $password = $this->password;
        $from     = $this->senderNumber;
        $to       = $this->mobile;
        $massage  = $this->message;

        if ( empty( $username ) || empty( $password ) ) {
            return false;
        }

        $to = implode( '-', $to );
        $to = str_ireplace( '+98', '0', $to );

        $data = [
            'username'    => $username,
            'password'    => $password,
            'destination' => $to,
            'source'      => $from,
            'message'     => $massage,
        ];

        $this->log_sms("ارسال پیامک: " . json_encode($data, JSON_UNESCAPED_UNICODE));

        $remote = wp_remote_get( 'https://panel.asanak.com/webservice/v1rest/sendsms?' . http_build_query( $data ) );

        $response_code = wp_remote_retrieve_response_code( $remote ); // دریافت کد وضعیت HTTP
        $response_body = wp_remote_retrieve_body( $remote ); // دریافت بدنه پاسخ

        // ثبت لاگ برای پاسخ دریافت‌شده از API
        $this->log_sms("کد وضعیت HTTP: $response_code | پاسخ دریافت‌شده: " . $response_body);

        // بررسی کد وضعیت 403 و نمایش پیام مناسب
        if ( $response_code == 403 || $this->is_ip_blocked($response_body) ) {
            $this->log_sms("خطای 403: دسترسی محدود شده است. احتمالاً IP شما مسدود شده است.");
            return "خطا: دسترسی محدود شده است. لطفاً بررسی کنید که IP شما در لیست مجاز API باشد.";
        }

        if ( preg_match( '/\[.*\]/is', (string) $response_body ) ) {
            return true; // موفقیت‌آمیز
        }

        return $response_body;
    }

    private function is_ip_blocked($response) {
        return stripos($response, '403 Forbidden') !== false || 
               stripos($response, 'Access Denied') !== false || 
               stripos($response, '<html') !== false; // بررسی وجود HTML در پاسخ
    }

    private function log_sms( $logMessage ) {
        $logFile = __DIR__ . '/asanak_sms.log'; // مسیر فایل لاگ
        $time = date('Y-m-d H:i:s'); // زمان ثبت لاگ
        error_log("[$time] $logMessage\n", 3, $logFile);
    }
}
